package it.pmsc.myapplication.casethree;


import android.content.Context;
import android.text.Html;
import android.text.Spanned;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;
import it.pmsc.myapplication.R;

import java.util.ArrayList;
import java.util.List;

public class InfoAdapter extends RecyclerView.Adapter<InfoAdapter.ViewHolder> {
    private List<InfoItem> items;


    public InfoAdapter (List<InfoItem> items){
        this.items=items;
    }

    @NonNull
    @Override
    public InfoAdapter.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.info3_item, parent, false);
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull InfoAdapter.ViewHolder holder, final int position) {
        InfoItem item = items.get(position);
        holder.imageView.setImageResource(item.getImageResource());
        holder.titleTextView.setText(item.getTitle());

        holder.descTextView.setText(Html.fromHtml(item.getDesc()));


        boolean isExpandable = items.get(position).isExpandable();
        holder.expandableLayout.setVisibility(isExpandable?View.VISIBLE:View.GONE);
    }

    @Override
    public int getItemCount() {
        return items.size();
    }


    public class ViewHolder extends RecyclerView.ViewHolder{
        ImageView imageView;
        TextView titleTextView;
        TextView descTextView;

        LinearLayout linearLayout;
        RelativeLayout expandableLayout;

        public ViewHolder(@NonNull View itemView){
            super(itemView);
            imageView = itemView.findViewById(R.id.imageView);
            descTextView = itemView.findViewById(R.id.desc_textview);
            titleTextView = itemView.findViewById(R.id.title_textview);
            linearLayout = itemView.findViewById(R.id.linear_layout);
            expandableLayout = itemView.findViewById(R.id.expandable_layout);

            linearLayout.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    InfoItem item = items.get(getAdapterPosition());
                    item.setExpandable(!item.isExpandable());
                    notifyItemChanged(getAdapterPosition());
                }
            });
        }
    }
}
