package it.pmsc.myapplication.mappe;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.inputmethodservice.Keyboard;
import android.location.Address;
import android.location.Geocoder;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.text.Editable;
import android.text.Html;
import android.util.Log;
import android.util.SparseArray;
import android.view.View;
import android.view.autofill.AutofillValue;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.Spinner;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.MapView;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.UiSettings;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import it.pmsc.myapplication.DBAccess;
import it.pmsc.myapplication.MainActivity;
import it.pmsc.myapplication.R;

public class MapActivity extends AppCompatActivity implements OnMapReadyCallback {
    private static final String MAP_VIEW_BUNDLE_KEY="MapViewBundleKey";
    private static final String REGIONE="Regione";
    private MapView mapView;
    private GoogleMap gmap;
    private DBAccess dbAccess;
    private List<Luogo> luoghi;
    private List<String> regioni;
    private Geocoder geocoder;
   // private LocationManager locationManager;
   // private LocationListener locationListener;
   // private LatLng posizioneUtente;
    Dialog dialog;
    ImageView close;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_map);
        getSupportActionBar().setTitle(R.string.menu_maps);

        geocoder = new Geocoder(MapActivity.this, Locale.getDefault());
        createMapView(savedInstanceState);

        dbAccess= new DBAccess();
        regioni= new ArrayList<>();
        regioni=dbAccess.leggiRegioni();

        if(regioni==null){
        //errore di connessione al DB
            dialog=new Dialog(this);
            dialog.setContentView(R.layout.dialog_errore);
            close=(ImageView) dialog.findViewById(R.id.close_popup);
            Button ok = dialog.findViewById(R.id.btn_accept);
            close.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    dialog.dismiss();
                }
            });
            ok.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    Intent i= new Intent(MapActivity.this, MainActivity.class);
                    startActivity(i);
                }
            });

            dialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
            dialog.show();



        }else {
            //Connessione DB OK
            //popolare spinner: si usa l'adapter per fare un'operazione di inflating (prendere i dati e riempire la view)
            //si passa l'activity(this), il layout di riga e i dati che si vogliono inserire

            final AutoCompleteTextView regioneTv = findViewById(R.id.regione);
            ArrayAdapter<String> adapter = new ArrayAdapter<String>(this,android.R.layout.select_dialog_item,regioni);
            regioneTv.setThreshold(1);
            regioneTv.setAdapter(adapter);

            SharedPreferences prefs = getSharedPreferences(REGIONE,Context.MODE_PRIVATE);
            String regionePreferita = prefs.getString(REGIONE, "");

            if(regionePreferita!=null){
                regioneTv.setText(regionePreferita);
                luoghi=dbAccess.leggiLocali(regionePreferita);

                if (luoghi == null) {
                    Log.i("LOCALE ", "ERRORE");
                    //Toast.makeText(this, "Non riesco a leggere i locali", Toast.LENGTH_LONG);
                } else {

                    for (int i = 0; i < luoghi.size(); i++) {
                        //per ogni indirizzo chiamo un asyncTask che fa geocoding e disegna
                        new GeocodingTask().execute(luoghi.get(i).getNome(),luoghi.get(i).getIndirizzo());
                    }

                }
            }

            //listener per ascoltare la selezione della regione da parte dell'utente
            regioneTv.setOnItemClickListener(new AdapterView.OnItemClickListener() {

                @Override
                public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                        InputMethodManager inputMethodManager = (InputMethodManager) getSystemService(Activity.INPUT_METHOD_SERVICE);
                        inputMethodManager.hideSoftInputFromWindow(getCurrentFocus().getWindowToken(), 0);

                    //salvataggio mensa selezionata
                    Editable regioneSelezionata = regioneTv.getText();
                    //salva nelle preferenze l'ultima regione selezionata
                    //la volta successiva la mappa si aprirà su quella
                    savePreferences(regioneSelezionata.toString());

                    luoghi=dbAccess.leggiLocali(regioneSelezionata.toString());
                    //cancello gli eventuali marker presenti prima di disegnare quelli nuovi
                    gmap.clear();
                    if (luoghi == null) {
                        Log.i("LOCALE ", "ERRORE");
                        //Toast.makeText(this, "Non riesco a leggere i locali", Toast.LENGTH_LONG);
                    } else {

                        for (int i = 0; i < luoghi.size(); i++) {
                            //per ogni indirizzo chiamo un asyncTask che fa geocoding e disegna
                            new GeocodingTask().execute(luoghi.get(i).getNome(),luoghi.get(i).getIndirizzo());
                        }

                    }
                }

            });
        }

        //posizioneUtente= new LatLng(50,1);
        }



        private class GeocodingTask extends AsyncTask<String,Void,LatLng>{
            String nome;
            String indirizzo;
            @Override
            protected LatLng doInBackground(String... strings) {
                nome=strings[0];
                indirizzo=strings[1];
                LatLng latLng=geocoding(indirizzo);
                return latLng;
            }

            @Override
            protected void onPostExecute(LatLng latLng) {
                super.onPostExecute(latLng);

                if(latLng!=null) {
                    MarkerOptions markerOptions = new MarkerOptions();
                    markerOptions.position(latLng); // imposto posizione marker
                    markerOptions.title(nome); // titolo marker
                    markerOptions.snippet(indirizzo);
                    if(nome.contains("Bar"))
                        markerOptions.icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_AZURE));
                    else if(nome.contains("Mensa"))
                        markerOptions.icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_RED));
                    gmap.addMarker(markerOptions);
                    //gmap.moveCamera(CameraUpdateFactory.newLatLng(city));
                    gmap.moveCamera(CameraUpdateFactory.newLatLngZoom(latLng, 8));
                }

            }
        }




    public void createMapView(Bundle savedInstanceState){

        Bundle mapViewBundle = null;
        if (savedInstanceState != null) {
            mapViewBundle = savedInstanceState.getBundle(MAP_VIEW_BUNDLE_KEY);
        }
        mapView = findViewById(R.id.mapView);
        mapView.onCreate(mapViewBundle);
        mapView.getMapAsync(this);
    }

    @Override
    public void onMapReady(GoogleMap googleMap) {
        gmap = googleMap;
        //specficare il tipo di mappa
        gmap.setMapType(GoogleMap.MAP_TYPE_NORMAL);

        UiSettings uiSettings = gmap.getUiSettings();
        uiSettings.setZoomControlsEnabled(true);
        uiSettings.setIndoorLevelPickerEnabled(true);
        uiSettings.setMyLocationButtonEnabled(true);
        uiSettings.setMapToolbarEnabled(true);
        gmap.setMinZoomPreference(5);

        //centra la mappa in Italia
        gmap.moveCamera(CameraUpdateFactory.newLatLng(new LatLng(42.1405011,10.7091384)));
    }

    public LatLng geocoding (String indirizzo){
        //dall'indirizzo della mensa si ottiene una coppia di coordinate
        List<Address> addresses;
        try{
            addresses= geocoder.getFromLocationName(indirizzo,1);
            Address location= addresses.get(0);
            return new LatLng(location.getLatitude(),location.getLongitude());

        } catch (IOException e) {
            //Errore geocoding --> java.io.IOException: grpc failed
            e.printStackTrace();
            //Toast.makeText(PrenotaActivity.this, "Impossibile visualizzare la posizione della mensa :(", Toast.LENGTH_LONG).show();
        }
        return null;
    }

    public void savePreferences(String regioneSelezionata) {
        SharedPreferences prefs = getSharedPreferences(REGIONE, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();

        if (regioneSelezionata != null) {
            editor.putString(REGIONE, regioneSelezionata);
            editor.commit();
        }
    }



    /*public void getLocation() {
        Log.i("PRENDO LOCATION","");
        locationManager = (LocationManager) this.getSystemService(Context.LOCATION_SERVICE);
        locationListener = new LocationListener() {
            @Override
            public void onLocationChanged(Location location) {
                Log.i("PosizioneUtente", location.getLatitude() + ", " + location.getLongitude());
                posizioneUtente= new LatLng(location.getLatitude(), location.getLongitude());
                gmap.moveCamera(CameraUpdateFactory.newLatLngZoom(posizioneUtente,15));
            }

            @Override
            public void onStatusChanged(String s, int i, Bundle bundle) {
            }

            @Override
            public void onProviderEnabled(String s) {
            }

            @Override
            public void onProviderDisabled(String s) {
            }
        };

    }

    public void verifyPermissions() {
        //controllare che i permessi del manifest siano di tipo ACCESS_FINE_LOCATION
        if(Build.VERSION.SDK_INT>=23){
            if (ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                //se l'utente non ha fornito i permessi li chiedo
                ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, 5);
            }else {
                //aggiornamento posizione utente quando si sposta di almeno 10 metri
                locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 10, locationListener);
            }
        }else {
            locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 10, locationListener);
        }

    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
    //effettua operazioni quando l'utente fornisce i permessi

        if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
    //sono stati forniti i permessi

            if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
    //richiede la posizione ogni volta che l'utente si è spostato di 10 m
                gmap.setMyLocationEnabled(true);
                locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 10, locationListener);
            }
        }
    }

    public void cercaIntornoAMe (View v){
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            Log.i("NON HO IL PERMESSO DI", "ANDARE AFFanculo");
            verifyPermissions();
            getLocation();
        }else{
            Log.i("HO IL PERMESSO DI", "ANDARE AFFanculo. sono qui: "+posizioneUtente);
            gmap.setMyLocationEnabled(true);
        }
        //gmap.moveCamera(CameraUpdateFactory.newLatLng(posizioneUtente));

    }
*/

    @Override
    public void onStart() {
        super.onStart();
        mapView.onStart();
    }

    @Override
    public void onStop() {
        super.onStop();
        mapView.onStop();
    }

    @Override
    public void onPause() {
        super.onPause();
        mapView.onPause();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        mapView.onDestroy();
    }

    //relativo alla memoria del telefono
    @Override
    public void onLowMemory() {
        super.onLowMemory();
        mapView.onLowMemory();
    }

}
