package it.pmsc.myapplication.shop;

import android.app.Activity;
import android.graphics.Color;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.Calendar;
import java.util.List;

import it.pmsc.myapplication.R;


public class ProductAdapter extends ArrayAdapter<Prodotto> {

    private final Activity context;
    private final List<Prodotto> listaProdotti;
    long oggi=Calendar.getInstance().getTimeInMillis();

    FirebaseDatabase database;
    DatabaseReference myRef;


    public ProductAdapter(Activity context, List<Prodotto> listaProdotti) {
        super(context, R.layout.product_layout);
        this.context = context;
        this.listaProdotti=listaProdotti;

        database = FirebaseDatabase.getInstance();
        myRef = database.getReference("Carrello");
    }


    public View getView(final int position, View convertView, ViewGroup parent) {
        LayoutInflater inflater = context.getLayoutInflater();
        View view= convertView;
        final ProductVH holder;
        try{
            if(convertView==null){
                view=inflater.inflate(R.layout.product_layout,null);
                holder=new ProductVH();
                holder.nomeTv=view.findViewById(R.id.nome_prodotto);
                holder.prezzoTv=view.findViewById(R.id.prezzo_prodotto);
                holder.scadenzaTv=view.findViewById(R.id.scadenza_prodotto);
                holder.meno=view.findViewById(R.id.meno);
                holder.piu=view.findViewById(R.id.piu);
                holder.qta=view.findViewById(R.id.qta);

                view.setTag(holder);
        }else{
                holder= (ProductVH) view.getTag();
            }

            //lettura da DB della quantità per riempire le relative textview
            myRef.child(listaProdotti.get(position).getNome())
                    .addValueEventListener(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    if(dataSnapshot.getValue(Prodotto.class)!=null) {
                        Prodotto p = dataSnapshot.getValue(Prodotto.class);
                        holder.qta.setText(p.getQta()+"");
                        Log.i("DB", p.getNome()+" "+p.getQta());
                    }else{
                        holder.qta.setText("0");
                    }
                }
                @Override
                public void onCancelled(DatabaseError error) {

                }
            });

            holder.nomeTv.setText(listaProdotti.get(position).getNome());
            holder.prezzoTv.setText("Prezzo: "+String.format("%.2f",listaProdotti.get(position).getPrezzo())+" €");


           holder.meno.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    //int quantita=listaProdotti.get(position).getQta();

                        int quantitaProdotto = Integer.parseInt(holder.qta.getText().toString());
                        quantitaProdotto-=1;

                        if(quantitaProdotto>0) {
                            listaProdotti.get(position).setQta(quantitaProdotto);
                            //scrittura nel DB
                            myRef.child(listaProdotti.get(position).getNome()).setValue(listaProdotti.get(position));
                        }else{
                            listaProdotti.get(position).setQta(0);
                            //rimozione dal db
                            myRef.child(listaProdotti.get(position).getNome()).removeValue();
                        }
                        notifyDataSetChanged();

                }
            });

            holder.piu.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    int quantitaProdotto = Integer.parseInt(holder.qta.getText().toString());
                    listaProdotti.get(position).setQta(quantitaProdotto+1);

                    //scrittura nel DB
                    myRef.child(listaProdotti.get(position).getNome()).setValue(listaProdotti.get(position));

                    notifyDataSetChanged();
                }
            });


            //mostriamo la data di scadenza solo per i prodotti che scandono entro 7 gg ed evidenziamo la riga
            long dataS=(listaProdotti.get(position).getDataScadenza()).getTime();
            long differenza=(dataS-oggi)/(1000*60*60*24);
            //Log.i("differenza",listaProdotti.get(position).getNome()+" "+differenza+"");
            if(differenza<7){
                //stampa la data di scadenza e evidenzia la riga
                holder.scadenzaTv.setText("Data Scadenza: "+listaProdotti.get(position).dataScadenzaToString());
                view.setBackgroundColor(context.getResources().getColor(R.color.colorProduct));
           }else{
                holder.scadenzaTv.setText("");
                view.setBackgroundColor(Color.TRANSPARENT);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }

        return view;

    }

    public static class ProductVH {
        public TextView nomeTv;
        public TextView prezzoTv;
        public TextView scadenzaTv;
        public ImageView meno;
        public ImageView piu;
        public TextView qta;
    }

    public int getCount(){
        return listaProdotti.size();
    }

    public Prodotto getItem(Prodotto position){
        return position;
    }

    public long getItemId(int position){
        return position;
    }

}