package it.pmsc.myapplication.shop;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;

import android.app.AlarmManager;
import android.app.Dialog;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.net.Uri;
import android.os.Bundle;
import android.os.SystemClock;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.AbsListView;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.List;
import it.pmsc.myapplication.DBAccess;
import it.pmsc.myapplication.MainActivity;
import it.pmsc.myapplication.R;

import static it.pmsc.myapplication.shop.Alarm.ACTION_ALARM;


public class ShopActivity extends AppCompatActivity {

    DBAccess dbAccess;
    List<String> categorieSpinner;
    List<String> categorieMS;
    List<String> categoriePP;
    List<Prodotto> prodotti;
    CheckBox piattiPronti;
    CheckBox miniShop;
    AutoCompleteTextView categoriaTv;
    ImageView drop_down;

    Dialog dialog;
    ImageView close;

    ListView listaProdotti;
    FloatingActionButton scroll;

    RelativeLayout relativeLayout;
    TextView cartTv;

    //database
    FirebaseDatabase database;
    DatabaseReference myRef;
    int numProdotti;

    //allarme
    PendingIntent alarmIntent;
    AlarmManager alarmManager;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_waste);
        getSupportActionBar().setTitle(R.string.menu_shop);

        database = FirebaseDatabase.getInstance();
        myRef = database.getReference("Carrello");

        //gestione notifica prodotti in scadenza
        Intent intentRicevuto = getIntent();
        String tipo = intentRicevuto.getStringExtra("notifica");
        Log.i("TIPO", tipo+"");

        dbAccess = new DBAccess();
        categoriePP = new ArrayList<>();
        categoriePP = dbAccess.leggiCategorieFromTipo("Piatti pronti");
        categorieMS = new ArrayList<>();
        categorieMS = dbAccess.leggiCategorieFromTipo("Mini-shop");

        if (categoriePP == null || categorieMS == null) {
            this.mostraErrore();

        } else {
            categorieSpinner = new ArrayList<>();
            categorieSpinner.add("Tutti i prodotti");
            prodotti = new ArrayList<>();

            piattiPronti = findViewById(R.id.piattiPronti);
            miniShop = findViewById(R.id.miniShop);
            categoriaTv = findViewById(R.id.categoriaTv);
            drop_down = findViewById(R.id.drop_down);
            categoriaTv.setThreshold(1);
            scroll = findViewById(R.id.scrollToTop);
            listaProdotti = (ListView) findViewById(R.id.listaProdotti);
            listaProdotti.setOnScrollListener(new AbsListView.OnScrollListener() {

                @Override
                public void onScrollStateChanged(AbsListView view, int scrollState) {
                }

                @Override
                public void onScroll(AbsListView view, int firstVisibleItem, int visibleItemCount, int totalItemCount) {
                    if (firstVisibleItem == 0)
                        scroll.hide();
                    else
                        scroll.show();
                }
            });



            ArrayAdapter<String> adapter1 = new ArrayAdapter<String>(this, android.R.layout.simple_dropdown_item_1line, categorieSpinner);
            categoriaTv.setAdapter(adapter1);

            if (tipo!=null && tipo.compareTo("mostraTuttiProdotti")==0) {
                categoriaTv.setText(categorieSpinner.get(0));
                prodotti=dbAccess.leggiTuttiProdotti();
                if (prodotti == null)
                    mostraErrore();
                else {
                    ProductAdapter adapter = new ProductAdapter(ShopActivity.this, prodotti);
                    listaProdotti.setAdapter(adapter);
                }
            }else if(tipo!= null && tipo.compareTo("mostraTuttiPiattiPronti")==0){
                piattiPronti.setChecked(true);
                categorieSpinner.addAll(categoriePP);
                prodotti=dbAccess.leggiTuttiPiattiPronti();
                if (prodotti == null)
                    mostraErrore();
                else {
                    ProductAdapter adapter = new ProductAdapter(ShopActivity.this, prodotti);
                    listaProdotti.setAdapter(adapter);
                }
            }

            drop_down.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    categoriaTv.showDropDown();
                }
            });


            piattiPronti.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (piattiPronti.isChecked()) {
                        categorieSpinner.addAll(categoriePP);
                    } else {
                        categorieSpinner.removeAll(categoriePP);
                    }
                    ArrayAdapter<String> adapter = new ArrayAdapter<String>(ShopActivity.this, android.R.layout.simple_dropdown_item_1line, categorieSpinner);
                    categoriaTv.setAdapter(adapter);
                    categoriaTv.setText("");
                }
            });

            miniShop.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (miniShop.isChecked()) {
                        categorieSpinner.addAll(categorieMS);
                    } else {
                        categorieSpinner.removeAll(categorieMS);
                    }
                    ArrayAdapter<String> adapter = new ArrayAdapter<String>(ShopActivity.this, android.R.layout.simple_dropdown_item_1line, categorieSpinner);
                    categoriaTv.setAdapter(adapter);
                    categoriaTv.setText("");
                }
            });


                categoriaTv.setOnItemClickListener(new AdapterView.OnItemClickListener() {

                @Override
                public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                    //chiude la tastiera
                    ((InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE))
                            .hideSoftInputFromWindow(categoriaTv.getWindowToken(),0);

                    String categ = categoriaTv.getText().toString();
                    if (categ.compareTo("Tutti i prodotti") == 0) {
                        prodotti = dbAccess.leggiTuttiProdotti();
                    } else
                        prodotti = dbAccess.leggiProdottiFromCategoria(categ);

                    if (prodotti == null)
                        mostraErrore();
                    else {
                        ProductAdapter adapter = new ProductAdapter(ShopActivity.this, prodotti);
                        listaProdotti.setAdapter(adapter);
                    }
                }
            });
        }
    }

    public void scrollToTop(View v) {
        listaProdotti.setSelectionAfterHeaderView();
    }

    public void closeAvviso(View v) {
        RelativeLayout ll = findViewById(R.id.avviso);
        ll.setVisibility(View.GONE);
    }

    public void mostraErrore() {
        //errore di connessione al DB
        dialog = new Dialog(this);
        dialog.setContentView(R.layout.dialog_errore);
        close = (ImageView) dialog.findViewById(R.id.close_popup);
        Button ok = dialog.findViewById(R.id.btn_accept);
        close.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dialog.dismiss();
            }
        });
        ok.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent i = new Intent(ShopActivity.this, MainActivity.class);
                startActivity(i);
            }
        });

        dialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        dialog.show();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.shopping_cart_menu, menu);

        relativeLayout=(RelativeLayout)menu.findItem(R.id.item_samplebadge).getActionView();
        cartTv=(TextView) relativeLayout.findViewById(R.id.counter);

        myRef.addValueEventListener(new ValueEventListener() {

            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                numProdotti=0;
                for(DataSnapshot prodotto : dataSnapshot.getChildren()){
                    numProdotti+=prodotto.getValue(Prodotto.class).getQta();
                }
                cartTv.setText(numProdotti+"");
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {

            }
        });

        relativeLayout.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent i = new Intent (ShopActivity.this, ShoppingCartActivity.class);
                if(numProdotti>0) {
                    //creazione della notifica per ricordare che c'è qualcosa nel carrello
                    creaAlarm();
                    //schedulazione dell'eliminazione da db se l'utente non conclude l'acquisto
                    new java.util.Timer().schedule(new java.util.TimerTask() {
                        @Override
                        public void run() {
                            if(numProdotti>0) {
                                eliminaAlarm();
                                myRef.removeValue();
                            }
                        }
                    },60*60*1000); //60*60*1000 dopo un'ora se non hai concluso l'acquisto il carrello viene svuotato
                }
                startActivity(i);
            }
        });

        return super.onCreateOptionsMenu(menu);
    }

    private void creaAlarm() {
        Log.i("Preferenze ALLARME" ,"CREO ALARM");
        Intent intentToFire = new Intent(getApplicationContext(), Alarm.class);
        intentToFire.setData(Uri.parse(ACTION_ALARM));
        intentToFire.setAction(ACTION_ALARM);

        alarmManager= (AlarmManager) getSystemService(Context.ALARM_SERVICE);
        alarmIntent= PendingIntent.getBroadcast(getApplicationContext(), 0,intentToFire,0);

        //dopo 30 minuti ricordo che ha un ordine in sospeso
        //chiedo se vuole completarlo o svuotare il carrello
        //Calendar calendar = Calendar.getInstance();
        //calendar.setTimeInMillis(calendar.getTimeInMillis()+(30*1000)); //30*60*1000
        long time = SystemClock.elapsedRealtime()+(45*1000);//sarebbe in realtà dopo 30minuti

        //calendar.set(Calendar.HOUR_OF_DAY, 17);
        //calendar.set(Calendar.MINUTE, 55);
        //calendar.set(Calendar.SECOND, 15);

        //se il dispositivo viene riavviato l'alarm viene cancellato
        //salvo l'orario nelle preferenze, al riavvio reimposto l'alarm in automatico
        //salvaInSP(Long.toString(calendar.getTimeInMillis()));
        salvaInSP(Long.toString(time));

        //alarmManager.set(AlarmManager.ELAPSED_REALTIME_WAKEUP, calendar.getTimeInMillis(), alarmIntent);
        alarmManager.set(AlarmManager.ELAPSED_REALTIME_WAKEUP, time, alarmIntent);
    }

    public void eliminaAlarm(){
        Intent intentToFire= new Intent(getApplicationContext(), Alarm.class);
        intentToFire.setData(Uri.parse(ACTION_ALARM));
        intentToFire.setAction(ACTION_ALARM);
        AlarmManager alarmManager= (AlarmManager) getSystemService(Context.ALARM_SERVICE);
        PendingIntent pendingIntent= PendingIntent.getBroadcast(getApplicationContext(), 0,intentToFire,0);
        alarmManager.cancel(pendingIntent);
    }

    public void salvaInSP(String millis){
        Log.i("Preferenze ALLARME" ,"SALVO ALLARME IN PREERENZE");
        SharedPreferences spref = getSharedPreferences("Allarme", Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = spref.edit();
        //SharedPreferences.Editor editor = getPreferences(MODE_PRIVATE).edit();
        editor.putString("Allarme", millis);
        editor.commit();
    }



}
