package it.pmsc.myapplication.shop;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.NotificationManagerCompat;

import android.app.AlarmManager;
import android.app.Dialog;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.net.Uri;
import android.os.Bundle;
import android.text.Html;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import it.pmsc.myapplication.MainActivity;
import it.pmsc.myapplication.R;

import static it.pmsc.myapplication.shop.Alarm.ACTION_ALARM;

public class ShoppingCartActivity extends AppCompatActivity {

    FirebaseDatabase database;
    DatabaseReference myRef;

    List<Prodotto> carrello;
    ListView listaCarrello;
    TextView totaleTv;
    TextView cashbackTv;

    Dialog dialog;
    double cashback;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_shopping_cart);
        getSupportActionBar().setTitle("Carrello");

        database = FirebaseDatabase.getInstance();
        myRef = database.getReference("Carrello");


        //gestione notifica carrello pieno
        Intent intentRicevuto = getIntent();
        String tipo = intentRicevuto.getStringExtra("notifica");
        //Log.i("TIPO", tipo+"");
        NotificationManagerCompat notificationManager = NotificationManagerCompat.from(this);
        notificationManager.cancelAll();

            if (tipo!=null && tipo.compareTo("svuotaCarrello")==0) {
                //svuota il carrello
                myRef.removeValue();
            }


        totaleTv=findViewById(R.id.totaleTv);
        cashbackTv = findViewById(R.id.cashbackTv);

        carrello = new ArrayList<>();

        myRef.addValueEventListener(new ValueEventListener() {

            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                carrello.clear();
                double totale=0;
                cashback=0;
                int numArticoli=0;
                long oggi = Calendar.getInstance().getTimeInMillis();
                for(DataSnapshot prodotto : dataSnapshot.getChildren()) {
                    Prodotto p = prodotto.getValue(Prodotto.class);
                    if (p.getQta() > 0) {
                        carrello.add(p);
                        totale += (p.getQta() * p.getPrezzo());
                        numArticoli += p.getQta();
                        Log.i("CARRELLO", prodotto.getValue(Prodotto.class).getNome());

                        long dataS = (p.getDataScadenza()).getTime();
                        long differenza = (dataS - oggi) / (1000 * 60 * 60 * 24);

                        if (differenza < 7) {
                            //stampa la data di scadenza e evidenzia la riga
                            cashback += (p.getPrezzo() * p.getQta());
                        }

                    }
                }
                totaleTv.setText(Html.fromHtml("Totale provvisorio ("+numArticoli+" articoli): <b>"+String.format("%.2f",totale)+" €</b>"));
                cashbackTv.setText(Html.fromHtml
                        ("Cashback ottenibile: <b>"+String.format("%.2f",(cashback*0.2))+" €</b>"));

                if(carrello.size()==0){
                    eliminaAlarm();
                    LinearLayout carrelloVuoto = findViewById(R.id.carrelloVuoto);
                    carrelloVuoto.setVisibility(View.VISIBLE);
                    LinearLayout layoutTotale = findViewById(R.id.layoutTotale);
                    layoutTotale.setVisibility(View.GONE);
                }
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {

            }
        });


            ShoppingCartAdapter adapter = new ShoppingCartAdapter(ShoppingCartActivity.this, carrello);
            listaCarrello = (ListView) findViewById(R.id.listaCarrello);
            listaCarrello.setAdapter(adapter);

    }

    public void concludiAcquisto(View v){
        //eliminare l'allarme per il carrello non concluso
        eliminaAlarm();

        //dialog campagna SaveMy
        if(cashback>0){
            mostraDialog();
        }else{
            //ha pagato, torna alla home
            Intent i = new Intent(ShoppingCartActivity.this, MainActivity.class);
            startActivity(i);
            //elimina il carrello da DB
            myRef.removeValue();
        }



    }

    public void eliminaAlarm(){
        Intent intentToFire= new Intent(getApplicationContext(), Alarm.class);
        intentToFire.setData(Uri.parse(ACTION_ALARM));
        intentToFire.setAction(ACTION_ALARM);
        AlarmManager alarmManager= (AlarmManager) getSystemService(Context.ALARM_SERVICE);
        PendingIntent pendingIntent= PendingIntent.getBroadcast(getApplicationContext(), 0,intentToFire,0);
        alarmManager.cancel(pendingIntent);
    }

    public void mostraDialog() {
        //errore di connessione al DB
        dialog = new Dialog(this);
        dialog.setContentView(R.layout.dialog_savemy);

        Button ok = dialog.findViewById(R.id.btn_ok);

        ok.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                //ha pagato, torna alla home
                Intent i = new Intent(ShoppingCartActivity.this, MainActivity.class);
                startActivity(i);
                //elimina il carrello da DB
                myRef.removeValue();
            }
        });

        dialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        dialog.show();
    }
}
