package it.pmsc.myapplication.shop;

import android.app.Activity;
import android.graphics.Color;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;

import java.util.Calendar;
import java.util.List;

import it.pmsc.myapplication.R;


public class ShoppingCartAdapter extends ArrayAdapter<Prodotto> {

    private final Activity context;
    private final List<Prodotto> carrello;

    FirebaseDatabase database;
    DatabaseReference myRef;

    long oggi=Calendar.getInstance().getTimeInMillis();


    public ShoppingCartAdapter(Activity context, List<Prodotto> carrello) {
        super(context, R.layout.cart_layout);
        this.context = context;
        this.carrello=carrello;

        database = FirebaseDatabase.getInstance();
        myRef = database.getReference("Carrello");
    }


    public View getView(final int position, View convertView, ViewGroup parent) {
        LayoutInflater inflater = context.getLayoutInflater();
        View view= convertView;
        final ProductVH holder;
        try{
            if(convertView==null){
                view=inflater.inflate(R.layout.cart_layout,null);
                holder=new ProductVH();
                holder.nomeTv=view.findViewById(R.id.nomeProdottoCarrello);
                holder.prezzoTv=view.findViewById(R.id.prezzoProdottoCarrello);
                holder.qtaTv=view.findViewById(R.id.qtaProdottoCarrello);
                holder.cestino=view.findViewById(R.id.cestino);

                view.setTag(holder);
            }else{
                holder= (ProductVH) view.getTag();
            }

            holder.nomeTv.setText(carrello.get(position).getNome());
            holder.prezzoTv.setText("Prezzo unitario: "+String.format("%.2f",carrello.get(position).getPrezzo())+" €");
            holder.qtaTv.setText("Quantità: "+carrello.get(position).getQta()+"");


            holder.cestino.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                        //scrittura nel DB
                        myRef.child(carrello.get(position).getNome()).removeValue();
                        carrello.remove(position);
                        notifyDataSetChanged();

                }
            });

            //evidenziamo la riga dei prodotti in scadenza che danno diritto al cashback
            long dataS=(carrello.get(position).getDataScadenza()).getTime();
            long differenza=(dataS-oggi)/(1000*60*60*24);

            if(differenza<7){
                //stampa la data di scadenza e evidenzia la riga
                view.setBackgroundColor(context.getResources().getColor(R.color.colorProduct));
            }else{
                view.setBackgroundColor(Color.TRANSPARENT);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }

        return view;

    }

    public static class ProductVH {
        public TextView nomeTv;
        public TextView prezzoTv;
        public TextView qtaTv;
        public ImageView cestino;
    }

    public int getCount(){
        return carrello.size();
    }

    public Prodotto getItem(Prodotto position){
        return position;
    }

    public long getItemId(int position){
        return position;
    }

}
